/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
#ifndef LINK_STATUS_H
#define LINK_STATUS_H

#include <set>
#include <string>

#include <boost/asio.hpp>
#include <boost/shared_ptr.hpp>

#include "link/statusnotifiercommand.h"

//-----------------------------------------------------------------------------
// LinkStatus
//-----------------------------------------------------------------------------

/**
 * @brief This class analyzes and notifies the link status, through keeping
 * track of the amount of hosts down.
 * Scope: one object for many hosts.
 */
class LinkStatus
{
public:
    LinkStatus(
            const int hosts_down_limit,
            const int link_up_interval_in_sec,
            const int link_down_interval_in_sec,
            const std::string &status_notifier_cmd
    );
    ~LinkStatus();

    void notify_host_up( const std::string &host_address );
    void notify_host_down( const std::string &host_address );

private:
    std::string log_prefix() const;

    //
    // Types
    //

    enum Status
    {
        Status_Up,
        Status_Down
    };

    enum NotificationStatus
    {
        NotificationStatus_NotReported,
        NotificationStatus_Reported
    };

#ifdef UNDER_TEST
public:
#endif
    //
    // Methods
    //

    bool add_host_up( const std::string &host_address );
    void add_host_down( const std::string &host_address );

    bool exceeded_host_down_limit() const;

    void notify_link_up();
    void notify_link_down();

    bool is_link_up_enough_time() const;
    bool is_link_down_enough_time() const;

    bool can_report_link_status() const;
    void set_link_status(
            const LinkStatus::Status new_link_status
    );

    //
    // Attributes
    //

    /// The maximum amount of hosts which can be down before sound the alarm
    const int HostsDownLimit;
    /// List of hosts down (obvious isn't it?)
    std::set<std::string> HostsDownList;
    /// Interval the link have to be stable in order to consider it is functional
    const int LinkUpIntervalInSec;
    /// Interval the link have to be down in order to consider it is non-functional
    const int LinkDownIntervalInSec;
    /// Keep track of the actual link status
    LinkStatus::Status CurrentLinkStatus;
    /// Indicates if the last link status change was notified
    LinkStatus::NotificationStatus CurrentNotificationStatus;
    /// When was the last time the status changed
    boost::posix_time::ptime TimeLinkStatusChanged;
    /// Command used to notify the status of the link
    StatusNotifierCommandItem StatusNotifierCmd;
    /// last reported status to prevent multiple calls of same status
    LinkStatus::Status LastReportedStatus;

};

//-----------------------------------------------------------------------------
// LinkStatusItem
//-----------------------------------------------------------------------------

typedef boost::shared_ptr<LinkStatus> LinkStatusItem;

#endif // LINK_STATUS_H
